///	ComponentFactory.cpp - Factory to instantiate all the components in the
///						   ScriptedMainWindow, according to what it reads from
///						   the xml file.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2004-2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------

#include <juce.h>

#include "ComponentFactory.h"

///---------------------------------------------------------------------------
ComponentFactory::ComponentFactory(Component *window, String filepath)
{
	this->window = window;
	this->filepath = filepath;
}

///---------------------------------------------------------------------------
ComponentFactory::~ComponentFactory()
{
	int i;

	for(i=0;i<array.size();i++)
		delete array[i]->object;
}

///---------------------------------------------------------------------------
void ComponentFactory::registerObject(FactoryObject *object, String& name)
{
	FactoryObjectStuff *tempObject = new FactoryObjectStuff;

	tempObject->object = object;
	tempObject->name = name;

	array.add(tempObject);
}

///---------------------------------------------------------------------------
void ComponentFactory::registerStringAttribute(FactoryObject *object,
											   String& name,
											   stringAttributeFunc *method)
{
	int i;
	stringEntry *tempAtt;

	for(i=0;i<array.size();i++)
	{
		if(array[i]->object == object)
		{
			tempAtt = new stringEntry;
			tempAtt->name = name;
			tempAtt->func = method;

			array[i]->stringAttributes.add(tempAtt);
			break;
		}
	}
}

///---------------------------------------------------------------------------
void ComponentFactory::registerDoubleAttribute(FactoryObject *object,
											   String& name,
											   doubleAttributeFunc *method)
{
	int i;
	doubleEntry *tempAtt;

	for(i=0;i<array.size();i++)
	{
		if(array[i]->object == object)
		{
			tempAtt = new doubleEntry;
			tempAtt->name = name;
			tempAtt->func = method;

			array[i]->doubleAttributes.add(tempAtt);
			break;
		}
	}
}

///---------------------------------------------------------------------------
void ComponentFactory::registerIntAttribute(FactoryObject *object,
											String& name,
											intAttributeFunc *method)
{
	int i;
	intEntry *tempAtt;

	for(i=0;i<array.size();i++)
	{
		if(array[i]->object == object)
		{
			tempAtt = new intEntry;
			tempAtt->name = name;
			tempAtt->func = method;

			array[i]->intAttributes.add(tempAtt);
			break;
		}
	}
}

///---------------------------------------------------------------------------
void ComponentFactory::registerBoolAttribute(FactoryObject *object,
											 String& name,
											 boolAttributeFunc *method)
{
	int i;
	boolEntry *tempAtt;

	for(i=0;i<array.size();i++)
	{
		if(array[i]->object == object)
		{
			tempAtt = new boolEntry;
			tempAtt->name = name;
			tempAtt->func = method;

			array[i]->boolAttributes.add(tempAtt);
			break;
		}
	}
}

///---------------------------------------------------------------------------
void ComponentFactory::createComponents()
{
	int i, j;
	XmlDocument *doc;
	XmlElement *rootElement;
	XmlElement *tempElement;
	String tempstr;
	bool tempb = true; //Used to set components visible or not, depending on their enabled attribute.

	doc = new XmlDocument(File(filepath));
    rootElement = doc->getDocumentElement();

	if(!rootElement)
	{
		tempstr = T("Oh No!  The JUCEScriptApplication XML file does not exist?\n");
		tempstr << filepath;
		tempstr << T("\n Please don't set the dogs on me!");
		AlertWindow::showMessageBox(AlertWindow::WarningIcon, T("XML Error"), tempstr);
		return;
	}

	if(!rootElement->hasTagName("JUCEScriptApplication"))
	{
		AlertWindow::showMessageBox(AlertWindow::WarningIcon,
									T("XML Error"),
									T("Oh No!  This is not a JUCEScriptApplication XML file...\nPlease don't hurt my mother - she's innocent I tell you!"));
		return;
	}

	//loop through every FactoryObject to set all their attributes from the xml file.
	for(i=0;i<array.size();i++)
	{
		tempElement = rootElement->getFirstChildElement();
		//loop through every xml element, looking for the one that matches the current (i) FactoryObject
		while(tempElement != 0)
		{
			if(tempElement->hasTagName(array[i]->name))
			{
				setupPosSize(array[i], tempElement);

				//loop through all the string attributes for this FactoryObject
				for(j=0;j<array[i]->stringAttributes.size();j++)
				{
					tempstr = array[i]->stringAttributes[j]->name;
					//set the appropriate string attribute, if it exists
					if(tempElement->hasAttribute(tempstr))
						array[i]->stringAttributes[j]->func(tempElement->getStringAttribute(tempstr));
				}

				//loop through all the double attributes for this FactoryObject
				for(j=0;j<array[i]->doubleAttributes.size();j++)
				{
					tempstr = array[i]->doubleAttributes[j]->name;
					//set the appropriate double attribute, if it exists
					if(tempElement->hasAttribute(tempstr))
						array[i]->doubleAttributes[j]->func(tempElement->getDoubleAttribute(tempstr));
				}

				//loop through all the int attributes for this FactoryObject
				for(j=0;j<array[i]->intAttributes.size();j++)
				{
					tempstr = array[i]->intAttributes[j]->name;
					//set the appropriate int attribute, if it exists
					if(tempElement->hasAttribute(tempstr))
						array[i]->intAttributes[j]->func(tempElement->getIntAttribute(tempstr));
				}

				//loop through all the bool attributes for this FactoryObject
				for(j=0;j<array[i]->boolAttributes.size();j++)
				{
					tempstr = array[i]->boolAttributes[j]->name;
					//set the appropriate bool attribute, if it exists.
					if(tempElement->hasAttribute(tempstr))
						array[i]->boolAttributes[j]->func(tempElement->getBoolAttribute(tempstr));
				}

				//Check if the component's supposed to be enabled or not.
				if(tempElement->hasAttribute(T("enabled")))
					array[i]->object->setEnabled(tempElement->getBoolAttribute(tempstr));

				//all the attributes have been set now, so instantiate the object
				array[i]->object->create();
				break;
			}
			tempElement = tempElement->getNextElement();
		}
	}

	//Loop through every FactoryObject to enable/disable the correct ones.
	//(have to do this here, because the window will have only just been created)
	for(i=0;i<array.size();i++)
	{
		tempb = array[i]->object->getEnabled();
		if(array[i]->name != "mainWindow")
			array[i]->object->getComponent()->setVisible(tempb);
	}

	//loop through every FactoryObject to instantiate all the actual components...
	//for(i=0;i<array.size();i++)
		//array[i]->object->create();
}

///---------------------------------------------------------------------------
void ComponentFactory::resizeComponents()
{
	int i;
	int tempx, tempy;
	float tempxf, tempyf;
	int tempw, temph;
	float tempwf, temphf;
	FactoryObject *tempObj;
	//char tempstr[256];

	tempxf = -1.0f;
	tempyf = -1.0f;

	for(i=0;i<array.size();i++)
	{
		//because the main window's already been resized by this point
		if(array[i]->name == T("mainWindow"))
			continue;

		//strcpy(tempstr, array[i]->name);
		//tempstr[0] = tempstr[0];

		tempObj = array[i]->object;

		//Get the object's position...
		tempx = tempObj->getX();
		if(tempx < 0) //If tempx is negative, it means it should be placed relative to the left edge of the window.
			tempx += window->getWidth();
		tempy = tempObj->getY();
		if(tempy < 0) //If tempy is negative, it means it should be placed relative to the bottom edge of the window.
			tempy += window->getHeight();

		//Make it so that absolute values take precedence over relative ones...
		if(tempx == -1)
		{
			tempxf = tempObj->getXRel();
			tempxf = (tempxf/100.0f)*static_cast<float>(window->getWidth());
		}
		else
			tempxf = static_cast<float>(tempx);
		if(tempy == -1)
		{
			tempyf = tempObj->getYRel();
			tempyf = (tempyf/100.0f)*static_cast<float>(window->getHeight());
		}
		else
			tempyf = static_cast<float>(tempy);

		//Get the object's size...
		tempw = tempObj->getWidth();
		if(tempw < 0) //If tempw is negative, it means it should be placed relative to the width of the window.
			tempw += window->getWidth();
		temph = tempObj->getHeight();
		if(temph < 0) //If temph is negative, it means it should be placed relative to the height of the window.
			temph += window->getHeight();
		//Make it so that absolute values take precedence over relative ones...
		if(tempw == -1)
		{
			tempwf = tempObj->getWidthRel();
			tempwf = (tempwf/100.0f)*static_cast<float>(window->getWidth());
		}
		else
			tempwf = static_cast<float>(tempw);
		if(temph == -1)
		{
			temphf = tempObj->getHeightRel();
			temphf = (temphf/100.0f)*static_cast<float>(window->getHeight());
		}
		else
			temphf = static_cast<float>(temph);
		if((tempxf != -1.0f)&&(tempyf != -1.0f)&&(tempwf != -1.0f)&&(temphf != -1.0f))
		{
			tempObj->getComponent()->setBounds(static_cast<int>(tempxf), static_cast<int>(tempyf), static_cast<int>(tempwf), static_cast<int>(temphf));
			//tempObj->getComponent()->setTopLeftPosition(static_cast<int>(tempxf), static_cast<int>(tempyf));
		}
	}
}

///---------------------------------------------------------------------------
void ComponentFactory::setupPosSize(FactoryObjectStuff *object, XmlElement *element)
{
	String tempstr;

	tempstr = T("x");
	if(element->hasAttribute(tempstr))
		object->object->setX(element->getIntAttribute(tempstr));
	else
	{
		tempstr = T("xRel");
		if(element->hasAttribute(tempstr))
			object->object->setXRel(element->getDoubleAttribute(tempstr));
	}
	tempstr = T("y");
	if(element->hasAttribute(tempstr))
		object->object->setY(element->getIntAttribute(tempstr));
	else
	{
		tempstr = T("yRel");
		if(element->hasAttribute(tempstr))
			object->object->setYRel(element->getDoubleAttribute(tempstr));
	}

	tempstr = T("width");
	if(element->hasAttribute(tempstr))
		object->object->setWidth(element->getIntAttribute(tempstr));
	else
	{
		tempstr = T("widthRel");
		if(element->hasAttribute(tempstr))
			object->object->setWidthRel(element->getDoubleAttribute(tempstr));
	}
	tempstr = T("height");
	if(element->hasAttribute(tempstr))
		object->object->setHeight(element->getIntAttribute(tempstr));
	else
	{
		tempstr = T("heightRel");
		if(element->hasAttribute(tempstr))
			object->object->setHeightRel(element->getDoubleAttribute(tempstr));
	}
}
