//	NPBPluginOrganiser.cpp - Keeps track of all the plugins loaded, and
//						     handles all the audio & control stuff too.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBPluginOrganiser.h"
#include "NPBPlugin.h"

//----------------------------------------------------------------------------
NPBPluginOrganiser::NPBPluginOrganiser():
Component(),
envelopeDecrement(0.0001f),
bpm(120.0f),
midiDeviceIndex(-1),
midiDevice(0),
directMode(false)
{
	audio = 0;
	NumChannels = 2;
	BufferSize = 128;
	InBuffer = 0;
	OutBuffer = 0;
	isRecording = false;
	recordWriter = 0;

	backColour = Colours::antiquewhite;
	pluginColour = Colours::lightgreen;
	pluginButtonColour = Colour(87, 176, 205);

	SetUpFactory();
}

//----------------------------------------------------------------------------
NPBPluginOrganiser::~NPBPluginOrganiser()
{
	if(midiDevice)
		delete midiDevice;

	if(audio)
		StopAudio();
	PluginArray.clear(true); //just in case
	if(recordWriter)
		delete recordWriter; //close the .wav file, if there is one...
	if(InBuffer) //Oops...  How did I miss this?
	{
		for(int i=0;i<NumChannels;i++)
		{
			delete [] InBuffer[i];
			delete [] OutBuffer[i];
		}
		for(i=0;i<2;i++)
			delete [] recordArray[i];
		delete [] InBuffer;
		delete [] OutBuffer;
		delete [] recordArray;
	}
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::paint(Graphics& g)
{
	g.setColour(Colour::greyLevel(0.4f));
    RectangleList visible;
    getVisibleArea(visible, false);
    visible.fill(g);

	GradientBrush gradientBrush (backColour.brighter(0.2f), 0.0f, 0.0f,
                                 backColour, (float)getWidth(), (float)getHeight(),
                                 true);
	g.setBrush (&gradientBrush);
	g.fillAll();

	g.setColour (Colours::black.withAlpha (0.25f));
    g.drawRect (0, 0, getWidth(), getHeight());
	g.setColour (Colours::black.withAlpha (0.125f));
    g.drawRect (1, 1, (getWidth()-2), (getHeight()-2));
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::actionListenerCallback(const String& m)
{
	
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::audioDeviceIOCallback(const float **inputChannelData,
											   int totalNumInputChannels,
											   float **outputChannelData,
											   int totalNumOutputChannels,
											   int numSamples)
{
	int i, j, k, l;

	if((totalNumOutputChannels == 0)||(totalNumInputChannels == 0))
		return;

	if(PluginArray.size() > 0)
	{
		l = 0;
		for(j=0;j<NumChannels;j++)
		{
			if(Inputs[j])
			{
				for(i=0;i<numSamples;i++)
				{
					InBuffer[l][i] = inputChannelData[j][i];
				}
				l++;
			}
		}

		for(i=0;i<(PluginArray.size()-1);i++)
		{
			PluginArray[i]->ProcessAudio(InBuffer, OutBuffer, numSamples);
			for(j=0;j<NumChannels;j++)
			{
				for(k=0;k<numSamples;k++)
					InBuffer[j][k] = OutBuffer[j][k];
			}
		}
		PluginArray[(PluginArray.size()-1)]->ProcessAudio(InBuffer, OutBuffer, numSamples);

		l = 0;
		for(j=0;j<NumChannels;j++)
		{
			if(Outputs[j])
			{
				for(i=0;i<numSamples;i++)
				{
					outputChannelData[j][i] = OutBuffer[l][i];
				}
				l++;
			}
		}
	}
	else //just copy input to output
	{
		l = 0;
		for(j=0;j<NumChannels;j++)
		{
			if(Inputs[j])
			{
				while(!Outputs[l]&&(l<NumChannels))
					l++;

				if(!Outputs[l]) //i.e. we've gone past the end of the array
					break;

				for(i=0;i<numSamples;i++)
				{
					outputChannelData[l][i] = inputChannelData[j][i];
				}
				l++;
			}
		}
	}

	//.wav recording
	if(isRecording)
	{
		l = 0;
		for(j=0;j<2;j++)
		{
			while(!Outputs[l]&&(l<NumChannels))
				l++;
			if(!Outputs[l]) //i.e. we've gone past the end of the array
				break;
			for(i=0;i<numSamples;i++)
			{
				//See JUCE docs for the significance of the 0x7fffffff value.				
				recordArray[j][i] = static_cast<int>(outputChannelData[l][i]*static_cast<float>(0x7fffffff));;
			}
		}
		recordWriter->write(const_cast<const int **>(recordArray), numSamples);
	}
	else if(recordWriter) //i.e. we're not recording, but we haven't deleted/closed the file yet...
	{
		delete recordWriter; //calling delete will close the file
		recordWriter = 0;
	}
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::joystickCallback(JoystickEvent& joystate, int index)
{
	int i;

	for(i=0;i<PluginArray.size();i++)
	{
		PluginArray[i]->joystickCallback(joystate, index);
	}
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::handleIncomingMidiMessage(MidiInput *source,
												   const MidiMessage &message)
{
	for(int i=0;i<PluginArray.size();i++)
		PluginArray[i]->addMIDIMessage(message);
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::mouseDown(const MouseEvent& e)
{
	int i;
	NPBPlugin *tempPlugin;

	if(e.mods.isRightButtonDown())
	{
		PopupMenu menu;

		for(i=0;i<Factory.size();i++)
			menu.addItem((i+1), Factory[i]->getMenuText());

		const int result = menu.show();

		if(result != 0)
		{
			PluginArray.add(Factory[result-1]->createInstance(this));
			if(!PluginArray.getLast()->getIsOk())
			{
				AlertWindow error(T("Plugin Error"), T("Something went wrong instantiating this plugin - it will now be deleted."), AlertWindow::WarningIcon);
				error.addButton(T("OK"), 0);
				error.runModalLoop();
				PluginArray.removeLast();
			}
			else
			{
				PluginArray.getLast()->setID(result);
				PluginArray.getLast()->setOrganiser(this);
				PluginArray.getLast()->setNumChannels(NumChannels);
				addChildComponent(PluginArray.getLast());
				PluginArray.getLast()->setVisible(true);
				PluginArray.getLast()->setTopLeftPosition((10*PluginArray.size())+((PluginArray.size()-1)*PluginArray.getLast()->getWidth()), 10);

				//Not very clever...
				if(result == 0)
				{
					tempPlugin = static_cast<NPBPlugin *>(PluginArray.getLast());
					tempPlugin->setEnvelopeDecrement(envelopeDecrement);
				}

				if(audio)
					PluginArray.getLast()->StartAudio();
			}
		}
	}
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::handleCommandMessage(int commandId)
{
	switch(commandId)
	{
		case DELETE_THIS_PLUGIN:
			for(unsigned long i=0;i<(unsigned long)PluginArray.size();i++)
			{
				if(PluginArray[i]->getDeleted())
				{
					PluginArray.remove(i);

					//Sort the remaining plugin's positions.
					for(unsigned long j=0;j<(unsigned long)PluginArray.size();j++)
					{
						PluginArray[j]->setTopLeftPosition((10*(j+1))+(j*PluginArray[j]->getWidth()), 10);
					}
					repaint();

					break;
				}
			}
			break;
	}
}

//----------------------------------------------------------------------------
bool NPBPluginOrganiser::filesDropped(const StringArray& filenames, int mouseX, int mouseY)
{
	int i;

	for(i=0;i<filenames.size();i++)
	{
		PluginArray.add(new NPBPlugin(this, filenames[i]));
		if(!PluginArray.getLast()->getIsOk())
		{
			AlertWindow error(T("Plugin Error"), T("Something went wrong instantiating this plugin - it will now be deleted."), AlertWindow::WarningIcon);
			error.addButton(T("OK"), 0);
			error.runModalLoop();
			PluginArray.removeLast();
		}
		else
		{
			PluginArray.getLast()->setID(1);
			PluginArray.getLast()->setOrganiser(this);
			PluginArray.getLast()->setNumChannels(NumChannels);
			addChildComponent(PluginArray.getLast());
			PluginArray.getLast()->setVisible(true);
			PluginArray.getLast()->setTopLeftPosition((10*PluginArray.size())+((PluginArray.size()-1)*PluginArray.getLast()->getWidth()), 10);
			if(audio)
				PluginArray.getLast()->StartAudio();
		}
	}

	return true;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setIOs(String soundcard, BitArray input, BitArray output)
{
	int inchan, outchan;

	NativeDesktopWindow* const w = getParentComponent()->getNativeWindow();
	ASIOAudioIODevice tempaudio(soundcard, w->getNativeHandle(), false);

	inchan = tempaudio.getOutputChannelNames().size();
	outchan = tempaudio.getOutputChannelNames().size();

	if(InBuffer)
	{
		for(int i=0;i<NumChannels;i++)
		{
			delete [] InBuffer[i];
			delete [] OutBuffer[i];
		}
		for(i=0;i<2;i++)
			delete [] recordArray[i];
		delete [] InBuffer;
		delete [] OutBuffer;
		delete [] recordArray;
	}

	//always make NumChannels the smallest number of the two
	if(inchan < outchan)
		NumChannels = inchan;
	else
		NumChannels = outchan;

	for(int i=0;i<PluginArray.size();i++)
	{
		PluginArray[i]->setNumChannels(NumChannels);
	}


	inchan = tempaudio.getNumBufferSizesAvailable();
	BufferSize = tempaudio.getBufferSizeSamples(inchan-1); //assuming the last entry is the biggest

	InBuffer = new float *[NumChannels];
	OutBuffer = new float *[NumChannels];
	recordArray = new int *[2];
	for(i=0;i<NumChannels;i++)
	{
		InBuffer[i] = new float[BufferSize];
		OutBuffer[i] = new float[BufferSize];
	}
	for(i=0;i<2;i++)		
		recordArray[i] = new int[BufferSize];
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::vstIdle()
{
	/*for(int i=0;i<PluginArray.size();i++)
		PluginArray[i]->vstIdle();*/
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::StartAudio()
{
	String tempstr;

	if(audio)
	{
		if(midiDevice)
			midiDevice->stop();
		if(audio->isPlaying())
			audio->stop();
		if(audio->isOpen())
			audio->close();
		delete audio;
		audio = 0;
	}
	NativeDesktopWindow* const w = getParentComponent()->getNativeWindow();
	audio = new ASIOAudioIODevice(Soundcard, w->getNativeHandle(), directMode);

	//tempstr = audio->open(Inputs, Outputs, 44100.0, audio->getBufferSizeSamples(0));
	tempstr = audio->open(Inputs, Outputs, 44100.0, audio->getDefaultBufferSize());
	if(!tempstr.isEmpty())
	{
		AlertWindow::showMessageBox(AlertWindow::WarningIcon,
									T("Audio Error"),
									T("Oh no!  Could not open the sound card!  I'm so sorry, please don't hit me!"));
		AlertWindow::showMessageBox(AlertWindow::WarningIcon,
									T("Error Description"),
									tempstr);
		delete audio;
		audio = 0;
	}
	else
	{
		for(int i=0;i<PluginArray.size();i++)
			PluginArray[i]->StartAudio();

		audio->start(this);

		if(midiDevice)
			midiDevice->start();
	}
}

//----------------------------------------------------------------------------
bool NPBPluginOrganiser::StartAudio(String soundcard, BitArray inputs, BitArray outputs)
{
	String tempstr;
	String tempstr2;

	if(audio)
	{
		if(midiDevice)
			midiDevice->stop();
		if(audio->isPlaying())
			audio->stop();
		if(audio->isOpen())
			audio->close();
		delete audio;
		audio = 0;
	}
	NativeDesktopWindow* const w = getParentComponent()->getNativeWindow();
	audio = new ASIOAudioIODevice(soundcard, w->getNativeHandle(), directMode);

	Soundcard = soundcard;
	Inputs = inputs;
	Outputs = outputs;

	tempstr = T("NPBPluginOrganiser::StartAudio(): Soundcard = ");
	tempstr << Soundcard << T(" directMode = ") << directMode;
	tempstr << T(" Inputs = ") << Inputs.toString(2) << T(" Outputs = ") << Outputs.toString(2);
	Logger::writeToLog(tempstr);

	//tempstr = audio->open(inputs, outputs, 44100.0, audio->getBufferSizeSamples(0));
	tempstr = audio->open(inputs, outputs, 44100.0, audio->getDefaultBufferSize());
	if(!tempstr.isEmpty())
	{
		AlertWindow::showMessageBox(AlertWindow::WarningIcon,
									T("Audio Error"),
									T("Oh no!  Could not open the sound card!  I'm so sorry, please don't hit me!"));
		AlertWindow::showMessageBox(AlertWindow::WarningIcon,
									T("Error Description"),
									tempstr);
		Logger::writeToLog(T("NPBPluginOrganiser::StartAudio(): Could not open soundcard."));
		tempstr2 = T("NPBPluginOrganiser::StartAudio(): Error = ");
		tempstr2 += tempstr;
		Logger::writeToLog(tempstr2);
		delete audio;
		audio = 0;
		return false;
	}
	else
		Logger::writeToLog(T("NPBPluginOrganiser::StartAudio(): Successfully opened soundcard."));

	for(int i=0;i<PluginArray.size();i++)
		PluginArray[i]->StartAudio();

	audio->start(this);
	if(midiDevice)
		midiDevice->start();

	return true;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::StopAudio()
{
	if(midiDevice)
		midiDevice->stop();

	for(int i=0;i<PluginArray.size();i++)
		PluginArray[i]->StopAudio();

	if(audio->isPlaying())
		audio->stop();
	if(audio->isOpen())
		audio->close();
	delete audio;
	audio = 0;

	/*if(recordWriter)
		delete recordWriter;
	recordWriter = 0;*/
}

//----------------------------------------------------------------------------
int NPBPluginOrganiser::getBlockSize()
{
	if(audio)
		return audio->getCurrentBufferSizeSamples();
	else
		return -1;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setDirectMode(bool val)
{
	directMode = val;
}

//----------------------------------------------------------------------------
StringArray NPBPluginOrganiser::getJoysticks()
{
	return ((NPBWindow *)getParentComponent())->getJoysticks();
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::clearOrganiser()
{
	PluginArray.clear(true);
	bpm = 120.0f;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::savePatch()
{
	int i, j, tempint;
	File filepath;
	XmlElement root("NPB");
	XmlElement *plugin;
	XmlElement *element;
	NPBMapping *map;
	String tempstr;

	tempstr = ((NPBWindow *)getParentComponent())->getPatchPath();

	FileChooser dlg(T("Save a patch..."), File(tempstr), T("*.npb"));

	if(dlg.browseForFileToSave())
	{
		filepath = dlg.getResult();
		tempstr = filepath.getFullPathName();
		tempint = tempstr.lastIndexOfChar(File::separator);
		tempstr = tempstr.dropLastCharacters(tempstr.length()-(tempint+1));
		((NPBWindow *)getParentComponent())->setPatchPath(tempstr);
	}
	else
		return;

	element = new XmlElement("Bpm");
	element->setAttribute("val", bpm);
	root.addChildElement(element);

	for(i=0;i<PluginArray.size();i++)
	{
		plugin = new XmlElement("Plugin");
		plugin->setAttribute("id", PluginArray[i]->getID());
		plugin->setAttribute("file", PluginArray[i]->getFile());
		plugin->setAttribute("acceptMIDI", PluginArray[i]->getAcceptMIDI());

		element = new XmlElement("Program");
		element->setAttribute("num", PluginArray[i]->getProgramNum());
		plugin->addChildElement(element);

		for(j=0;j<PluginArray[i]->getNumParameters();j++)
		{
			element = new XmlElement("Parameter");
			element->setAttribute("num", j);
			element->setAttribute("val", PluginArray[i]->getParameter(j));
			plugin->addChildElement(element);
		}

		for(j=0;j<PluginArray[i]->getNumMappings();j++)
		{
			element = new XmlElement("Mapping");
			map = PluginArray[i]->getMapping(j);
			element->setAttribute("joyindex", map->JoyIndex);
			element->setAttribute("joycontrol", map->JoyControl);
			element->setAttribute("index", map->Index);
			element->setAttribute("minval", map->MinVal);
			element->setAttribute("maxval", map->MaxVal);
			element->setAttribute("param", map->PluginParameter);
			element->setAttribute("paramType", map->parameterType);
			element->setAttribute("MIDI", map->MIDI);
			element->setAttribute("MIDINote", map->MIDINote);
			plugin->addChildElement(element);
		}

		root.addChildElement(plugin);
	}

	root.writeToFile(filepath, String::empty);
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::loadPatch()
{
	XmlDocument *doc;
	XmlElement *root;
	XmlElement *plugin;
	XmlElement *element;
	NPBMapping *map;
	String tempstr;

	tempstr = ((NPBWindow *)getParentComponent())->getPatchPath();

	PluginArray.clear(true);

	FileChooser dlg(T("Load a patch..."), File(tempstr), T("*.npb"));

	if(dlg.browseForFileToOpen())
		doc = new XmlDocument(File(dlg.getResult()));
	else
		return;

	root = doc->getDocumentElement();

	if(root->hasTagName("NPB"))
	{
		plugin = root->getFirstChildElement();

		if(plugin->hasTagName("Bpm"))
		{
			setBpm(static_cast<float>(plugin->getDoubleAttribute("val")));
			plugin = plugin->getNextElement();
		}

		while(plugin != 0)
		{
			if(plugin->hasTagName("Plugin"))
			{
				//first create the plugin...
				if(plugin->getIntAttribute("id") == 1)
				{
					PluginArray.add(new NPBPlugin(this, plugin->getStringAttribute("file")));
					if(!PluginArray.getLast()->getIsOk())
					{
						AlertWindow error(T("Plugin Error"), T("Something went wrong instantiating this plugin - it will now be deleted."), AlertWindow::WarningIcon);
						error.addButton(T("OK"), 0);
						error.runModalLoop();
						PluginArray.removeLast();
					}
					else
					{
						PluginArray.getLast()->setID(1);
						PluginArray.getLast()->setOrganiser(this);
						PluginArray.getLast()->setNumChannels(NumChannels);
						addChildComponent(PluginArray.getLast());
						PluginArray.getLast()->setVisible(true);
						PluginArray.getLast()->setTopLeftPosition((10*PluginArray.size())+((PluginArray.size()-1)*PluginArray.getLast()->getWidth()), 10);
						if(audio)
							PluginArray.getLast()->StartAudio();
					}
				}
				else
				{
					PluginArray.add(Factory[plugin->getIntAttribute("id")-1]->createInstance(this));
					if(!PluginArray.getLast()->getIsOk())
					{
						AlertWindow error(T("Plugin Error"), T("Something went wrong instantiating this plugin - it will now be deleted."), AlertWindow::WarningIcon);
						error.addButton(T("OK"), 0);
						error.runModalLoop();
						PluginArray.removeLast();
					}
					else
					{
						PluginArray.getLast()->setID(plugin->getIntAttribute("id"));
						PluginArray.getLast()->setOrganiser(this);
						PluginArray.getLast()->setNumChannels(NumChannels);
						addChildComponent(PluginArray.getLast());
						PluginArray.getLast()->setVisible(true);
						PluginArray.getLast()->setTopLeftPosition((10*PluginArray.size())+((PluginArray.size()-1)*PluginArray.getLast()->getWidth()), 10);
						if(audio)
							PluginArray.getLast()->StartAudio();
					}
				}

				//Whether it should accept MIDI or not.
				PluginArray.getLast()->setAcceptMIDI(plugin->getBoolAttribute("acceptMIDI"));

				element = plugin->getFirstChildElement(); //=Program
				//now set the program...
				PluginArray.getLast()->setProgramNum(element->getIntAttribute("num"));

				while(element != 0)
				{
					if(element->hasTagName("Parameter"))
					{
						PluginArray.getLast()->setParameter(element->getIntAttribute("num"), (float)element->getDoubleAttribute("val"));
					}
					else if(element->hasTagName("Mapping"))
					{
						map = new NPBMapping;
						map->JoyIndex = element->getIntAttribute("joyindex");
						map->JoyControl = (JoystickEventType)element->getIntAttribute("joycontrol");
						map->Index = element->getIntAttribute("index");
						map->MinVal = (float)element->getDoubleAttribute("minval");
						map->MaxVal = (float)element->getDoubleAttribute("maxval");
						map->PluginParameter = element->getIntAttribute("param");
						map->parameterType = element->getBoolAttribute("paramType");
						map->MIDI = element->getIntAttribute("MIDI");
						map->MIDINote = static_cast<char>(element->getIntAttribute("MIDINote"));

						PluginArray.getLast()->setMapping(map);
					}

					element = element->getNextElement();
				}
			}

			plugin = plugin->getNextElement();
		}
	}

	delete doc;
}

//----------------------------------------------------------------------------
bool NPBPluginOrganiser::startRecording(File &file)
{
	WavAudioFormat tempformat;
	FileOutputStream *recordStream;

	recordStream = new FileOutputStream(file);
	if(recordStream->areAnyErrors())
		return false;

	if(recordWriter)
		delete recordWriter;

	recordWriter = tempformat.createWriterFor(recordStream,
											  44100.0,
											  2,
											  16,	//I'm assuming it's ok not to check whether it supports 16-bit?
											  StringPairArray(), //i.e. empty
											  0);

	isRecording = true;
	return isRecording;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::stopRecording()
{
	if(!isRecording)
		return;	//Shouldn't ever get here, but just in case...

	isRecording = false; //Tell audio callback to stop recording.
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setBackColour(Colour col)
{
	backColour = col;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setPlugColour(Colour col)
{
	pluginColour = col;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setPlugButtonColour(Colour col)
{
	pluginButtonColour = col;
}

//----------------------------------------------------------------------------
NPBPluginBase *NPBPluginOrganiser::findPlugin(AEffect *plugin)
{
	int i;

	for(i=0;i<PluginArray.size();i++)
	{
		if(PluginArray[i]->getActualPlugin() == plugin)
			return PluginArray[i];
	}
	return 0;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setEnvelopeDecrement(float val)
{
	int i;
	NPBPlugin *tempPlugin;

	envelopeDecrement = val;

	for(i=0;i<PluginArray.size();i++)
	{
		//Okay, this isn't particularly smart...
		if(PluginArray[i]->getID() == 0)
		{
			tempPlugin = static_cast<NPBPlugin *>(PluginArray[i]);
			tempPlugin->setEnvelopeDecrement(envelopeDecrement);
		}
	}
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setBpm(float val)
{
	bpm = val;
}

//----------------------------------------------------------------------------
void NPBPluginOrganiser::setMIDIInputDevice(int index)
{
	midiDeviceIndex = index;

	midiDevice = MidiInput::openDevice(midiDeviceIndex, this);
}
